;
; Verifying the treaps package
;
; Modified for Chicken Scheme and the testeez package by Ivan Raikov
;
;
; $Id: vtreap.scm,v 1.2 2002/11/14 04:45:26 oleg Exp oleg $


(require-extension srfi-13)
(require-extension testeez)
(require-extension treap)

(define-macro (++! x) `(set! ,x (fx+ 1 ,x)))
(define-macro (++ x) `(fx+ 1 ,x))
(define-macro (--! x) `(set! ,x (fx- ,x 1)))
(define-macro (-- x) `(fx- ,x 1))


(define (treap-test:sort)
  (testeez "--> Sorting of a set of numbers via a treap"
	   
	   (test-define "" min-key -1) 
	   (test-define "" max-key 10)
	   
	   (test-define "" treap (make-treap (lambda (x y) (- x y))))
	   
	   (test-define "a hard-wired association between a key and a value"
			compute-assoc (lambda (key) (cons key (++ key))))
	   
	   (test/equal "" (treap 'empty?) #t)
	   (test/equal "" (zero? (treap 'size))  #t)
	   (test/equal "" (zero? (treap 'depth)) #t)
	   
	   (test-eval (string-concatenate (list "loading a sequence [" 
						(number->string min-key) ", " 
						(number->string max-key) "] in ascending order"))
		      (do ((i min-key (++ i))) ((> i max-key))
			(testeez (test/equal "" ((treap 'put!) i (cdr (compute-assoc i))) #f))))
	   
	   (test-eval "treap nodes: " (treap 'debugprint))
	   (test-eval "treap depth: " (treap 'depth))
	   
	   (test/equal "" (treap 'size) (++ (- max-key min-key)))
	   (test/equal "" (treap 'empty?)  #f)
	   (test/equal "" (treap 'get-min) (compute-assoc min-key))
	   (test/equal "" (treap 'get-max) (compute-assoc max-key))
	   (test/equal "" (treap 'get-min) ((treap 'get) min-key))
	   (test/equal "" (treap 'get-max) ((treap 'get) max-key))
	   
	   (test/equal "" ((treap 'get) (++ min-key))  (compute-assoc (++ min-key)))
	   (test/equal "" ((treap 'get) (++ min-key) #f)  (compute-assoc (++ min-key)))
	   
	   (test/equal "check looking up of non-existing keys" (not ((treap 'get) (-- min-key) #f)) #t)
	   (test/equal "" ((treap 'get) (++ max-key) (lambda () 1)) 1)
	   
	   ;;  (must-have-failed ((treap 'get) (-- min-key)))
	   
	   (test-eval "clear the treap" (treap 'clear!))
	   
	   (test/equal "" (treap 'empty?) #t)
	   (test/equal "" (zero? (treap 'size))  #t)
	   (test/equal "" (zero? (treap 'depth)) #t)
	   
	   ;;  (must-have-failed ((treap 'get) min-key))
	   ;;  (must-have-failed (treap 'get-min))
	   ;;  (must-have-failed (treap 'get-max))
	   
	   (test-eval "reloading the same seq in descending order" 
		      (do ((i max-key (-- i))) ((< i min-key))
			(testeez (test/equal "" ((treap 'put!) i (cdr (compute-assoc i))) #f))))

	   (test-eval "treap nodes: " (treap 'debugprint))
	   (test-eval "treap depth: " (treap 'depth))

	   (test/equal "" (treap 'get-min) (compute-assoc min-key))
	   (test/equal "" (treap 'delete-min!) (compute-assoc min-key))
	   (test/equal "" ((treap 'get) min-key #f) #f)
	   (test/equal "" (treap 'get-min) (compute-assoc (++ min-key)))
	   
	   (test/equal "" (treap 'get-max) ((treap 'get) max-key))
	   (test/equal "" (treap 'delete-max!) (compute-assoc max-key))
	   (test/equal "" ((treap 'get) max-key #f) #f)
	   (test/equal "" (treap 'get-max)  ((treap 'get) (-- max-key)))
	   (test/equal "" (treap 'size)     (+ -2 (++ (- max-key min-key))))
	   
	   (test-eval "remove the remaining treap elements one by one, from min to max" 
		      (do ((i (++ min-key) (++ i))) ((> i (-- max-key)))
			(testeez (test/equal "" ((treap 'get) i #f) (compute-assoc i))
				 (test/equal "" (treap 'delete-min!) (compute-assoc i))
				 (test/equal "" ((treap 'get) i #f) #f))))
	   
	   (test/equal "" (treap 'empty?) #t)
	   (test/equal "" (zero? (treap 'size))  #t)
	   (test/equal "" (zero? (treap 'depth)) #t)
	   
	   ;;  (must-have-failed ((treap 'for-each-ascending) (lambda (association) #t)))
	   ;;  (must-have-failed ((treap 'for-each-descending) (lambda (association) #t)))
	   
	   (test-eval "loading the treap again in a \"random\" order" 
		      (do ((i min-key) (j max-key) (direction #t (not direction)))
			  ((< j i))
			(cond
			 (direction
			  (testeez (test/equal "" ((treap 'put!) i (cdr (compute-assoc i))) #f))
			  (++! i))
			 (else
			  (testeez (test/equal "" ((treap 'put!) j (cdr (compute-assoc j))) #f))
			  (--! j)))))
	   
	   (test-define "" a-key (quotient (+ min-key max-key) 2))

	   (test-eval (string-concatenate (list "checking putting and deleting of an assoc with key " 
						(number->string a-key)))
		      (let* ((old-assoc (compute-assoc a-key))
			     (new-assoc (cons a-key #\a)))
			(testeez (test/equal "" ((treap 'get) a-key) old-assoc)
				 (test/equal "" ((treap 'put!) a-key (cdr new-assoc)) old-assoc)
				 (test/equal "" ((treap 'get) a-key) new-assoc)
				 (test/equal "" ((treap 'delete!) a-key) new-assoc)
				 (test/equal "" ((treap 'delete!) a-key #f) #f)
				 (test/equal "" ((treap 'put!) a-key (cdr old-assoc)) #f)
				 (test/equal "" ((treap 'put!) a-key (cdr old-assoc)) old-assoc)
				 (test/equal "" ((treap 'get) a-key) old-assoc))))
			
	   (test/equal "" (treap 'size) (++ (- max-key min-key)))

	   (test-eval "treap depth: " (treap 'depth))

	   (test/equal "" (treap 'empty?) #f)

  (test-eval "checking traversing in the ascending order" 
	     (let ((expected-key min-key))
	       ((treap 'for-each-ascending)
		(lambda (association)
		  (testeez (test/equal "" association (compute-assoc expected-key)))
		  (++! expected-key)))
	       (testeez (test/equal "" expected-key (++ max-key)))))

  (test-eval "checking traversing in the descending order"
	     (let ((expected-key max-key))
	       ((treap 'for-each-descending)
		(lambda (association)
		  (testeez (test/equal "" association (compute-assoc expected-key)))
		  (--! expected-key)))
	       (testeez (test/equal "" expected-key (-- min-key)))))))


(define (treap-test:map)
  (testeez "--> Build a treap to map from digit strings to the numbers "

	   (test-define "" lo 1)
	   (test-define "" hi 150)
	   (test-define "" treap (make-treap (lambda (x y) (if (string<? x y) -1 (if (string>? x y) 1 0)))))

	   (test-define "" min-key #f)
	   (test-define "" max-key #f)

	   (test/equal "" (treap 'empty?) #t)
	   (test/equal "" (zero? (treap 'size))  #t)
	   (test/equal "" (zero? (treap 'depth)) #t)

	   (test-eval (string-concatenate (list "loading a sequence [" 
						(number->string lo) "," 
						(number->string hi)
						"] in an ascending number order"))
		      (do ((i lo (++ i))) ((> i hi))
			(let ((key (number->string i)))
			  (testeez (test/equal "" ((treap 'put!) key i) #f))
			  (cond
			   ((not min-key) (set! min-key key) (set! max-key key))
			   ((string<? key min-key) (set! min-key key))
			   ((string>? key max-key) (set! max-key key))))))

	   (test-eval "treap depth: " (treap 'depth))

	   (test/equal "" (treap 'size) (++ (- hi lo)))
	   (test/equal "" (treap 'empty?) #f)
	   (test/equal "" (treap 'get-min) (cons min-key (string->number min-key)))
	   (test/equal "" (treap 'get-max) (cons max-key (string->number max-key)))
	   (test/equal "" (treap 'get-min) ((treap 'get) min-key))
	   (test/equal "" (treap 'get-max) ((treap 'get) max-key))

	   (test-define "" a-key-val (quotient (+ lo hi) 2))
	   (test-define "" a-key     (number->string a-key-val))

	   (test-eval (string-concatenate (list "checking putting and deleting of an assoc with key " a-key))
		      (let* ((old-assoc (cons a-key a-key-val))
			     (new-assoc (cons a-key #\a)))
			(testeez (test/equal "" ((treap 'get) a-key) old-assoc)
				 (test/equal "" ((treap 'put!) a-key (cdr new-assoc)) old-assoc)
				 (test/equal "" ((treap 'get) a-key) new-assoc)
				 (test/equal "" ((treap 'delete!) a-key) new-assoc)
				 (test/equal "" ((treap 'delete!) a-key #f) #f)
				 (test/equal "" ((treap 'put!) a-key (cdr old-assoc)) #f)
				 (test/equal "" ((treap 'put!) a-key (cdr old-assoc)) old-assoc)
				 (test/equal "" ((treap 'get) a-key) old-assoc))))
  
	   (test/equal "" (treap 'size) (++ (- hi lo)))

	   (test-define "" values-to-print 15) 
	   
	   (test-eval (string-concatenate (list "printing out first " 
						(number->string values-to-print)
						" associations in the descending\n"
						"\torder of their string keys (but not the values themselves!)"))
		      (let* ((printed 0))
			((treap 'for-each-descending)
			 (lambda (association)
			   (if (zero? printed)
			       (testeez (test/equal "" association (cons max-key (string->number max-key)))))
			   (if (< printed values-to-print)
			       (print "\t\t" association #\newline))
			   (++! printed)))
		      (testeez (test/equal "" printed (treap 'size)))))))
(if (treap-test:sort)
    (treap-test:map))
